(*
    ShowSpeed.applescript
    ShowSpeed - add the speed recorded by GPS to a movie.

    Created by Maarten Sneep on 15-08-08.
    Copyright © 2008 Maarten Sneep. 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*)

on awake from nib theObject
	set object_name to name of theObject as Unicode text
	if object_name = "main" then
		tell theObject
			set content of control "time_picker" of box "when" to (current date)
		end tell
		tell application "QuickTime Player"
			set ignore auto play to true
			set show welcome movie automatically to false
			set QTPro to QuickTime Pro installed
		end tell
		if not QTPro then
			tell me to printToLog("QuickTime Pro is not available")
			beep
			display alert (localized string "ShowSpeedQuickTimeProMissing") message (localized string "ShowSpeedQuickTimeProMissingMessage") attached to window "main"
		end if
	end if
end awake from nib

on clicked theObject
	set gpx_url to my getFileURL("gpx_dropbox")
	set movie_url to my getFileURL("movie_dropbox")
	if gpx_url = "file://localhost/" or movie_url = "file://localhost/" then
		tell me to printToLog("One or more files are missing")
		beep
		display alert (localized string "ShowSpeedSomeFilesMissing") message (localized string "ShowSpeedSomeFilesMissingMessage") attached to window "main"
		return
	end if
	
	set datestr to my getDateString()
	set timeIsBeginOfMovie to my timeIsBeginOfMovie()
	set titleDisplayDuration to my getTitleDuration()
	set mytitle to my getTitle()
	set rider to my getRider()
	set finale to my getFinale()
	set gpxRef to my fileURLtoPOSIXpath(gpx_url)
	set movieRef to my fileURLtoPOSIXpath(movie_url)
	
	tell application "QuickTime Player"
		open movieRef
		set theSize to natural dimensions of document 1
		
		set theDuration to duration of document 1
		set theTimeScale to time scale of document 1
		set theLength to theDuration / theTimeScale
		
		set durationStr to my numToStr(theLength)
	end tell
	
	set cmd to "python " & getPythonScript() & " "
	if timeIsBeginOfMovie then
		set cmd to cmd & "--movie-start=" & datestr & " "
	else
		set cmd to cmd & "--movie-end=" & datestr & " "
	end if
	set cmd to cmd & "--size=" & ((first item of theSize) as string) & "," & ((second item of theSize) as string) & " "
	set cmd to cmd & "--duration=" & durationStr & " "
	set cmd to cmd & "--race-start=" & (titleDisplayDuration as string) & " "
	set cmd to cmd & "--title=" & quoted form of mytitle & " "
	set cmd to cmd & "--rider=" & quoted form of rider & " "
	set cmd to cmd & "--finale=" & quoted form of finale & " "
	set cmd to cmd & quoted form of gpxRef & " "
	set movieTextRef to ((characters 1 through ((count gpxRef) - 4) in gpxRef) as string) & ".txt"
	set cmd to cmd & quoted form of movieTextRef
	tell me to printToLog(cmd)
	try
		set theReply to do shell script cmd
		log theReply
	on error theError
		log theError
		display alert (localized string "ShowSpeedGPXExtractionFailed") message (localized string "ShowSpeedGPXExtractionFailedMessage") attached to window "main"
		return
	end try
	tell application "QuickTime Player"
		open movieTextRef
		select all document 1
		copy document 1
		close document 1
		rewind document 1
		add document 1 without scaled
		rewind document 1
	end tell
	display alert (localized string "ShowSpeedConversionFinished") message (localized string "ShowSpeedConversionFinishedMessage") attached to window "main"
end clicked

on alert ended theObject with reply theReply
	tell application "QuickTime Player" to activate
end alert ended

on numToStr(v)
	set d to text item delimiters
	set text item delimiters to (0.0 as string)'s item 2
	set v to (v as string)'s text items
	set text item delimiters to "."
	set v to v as string
	set text item delimiters to d
	v
end numToStr

on getPythonScript()
	set mainPyScript to quoted form of ((resource path of main bundle) & "/makeQTtext.py")
end getPythonScript

on fileURLtoPOSIXpath(URLstring)
	set helperPyScript to quoted form of ((resource path of main bundle) & "/file_url_to_POSIX_path.py")
	set cmd to "python " & helperPyScript & " " & (URLstring as string)
	return (do shell script cmd)
end fileURLtoPOSIXpath

on should quit after last window closed theObject
	return true
end should quit after last window closed

on timeIsBeginOfMovie()
	return (current column of matrix "BeginEnd" of box "when" of window "main" is equal to 1)
end timeIsBeginOfMovie

on getDateString()
	set myDate to contents of control "time_picker" of box "when" of window "main"
	set myResult to formatNumber(((year of myDate) as integer), 4) & "-"
	set myResult to myResult & formatNumber(((month of myDate) as integer), 2) & "-"
	set myResult to myResult & formatNumber((day of myDate) as integer, 2) & "T"
	set myResult to myResult & formatNumber(((hours of myDate) as integer) - ((time to GMT) / 3600) as integer, 2) & ":"
	set myResult to myResult & formatNumber((minutes of myDate) as integer, 2) & ":"
	set myResult to myResult & formatNumber((seconds of myDate) as integer, 2) & "Z"
	return myResult
end getDateString

on formatNumber(num, len)
	set myResult to (num as string)
	if length of myResult < len then
		set myResult to "0" & formatNumber(num, len - 1)
	end if
	return myResult
end formatNumber

on getFileURL(dropbox)
	set str to (string value of control (dropbox as string) of box "files" of window "main")
	return str
end getFileURL

on getTitleDuration()
	return (integer value of text field "titleShowSeconds" of box "titles" of window "main")
end getTitleDuration

on getTitle()
	return (string value of text field "title" of box "titles" of window "main") as Unicode text
end getTitle

on getRider()
	return (string value of text field "rider" of box "titles" of window "main") as Unicode text
end getRider

on getFinale()
	return (string value of text field "finale" of box "titles" of window "main") as Unicode text
end getFinale

on printToLog(str)
	log str
end printToLog